/*
  BV4627 (I2C) 8 way relay from ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "Arduino.h"
#include "bv4627_I.h"
 
// **************************************************************
// **************************************************************
BV4627_I::BV4627_I(char i2adr):I2c_bv(i2adr)
{
  _i2adr=i2adr;  
}

// ==============================================================
// I2C utilities
// ==============================================================
// ==============================================================
// **************************************************************
// gets a 16 bit value from the i2c bus
// **************************************************************
int BV4627_I::i2_get16bit()
{
int rv;
  Wire.requestFrom(_i2adr, 2); // returns 2 bytes
  rv=Wire.read()*256; // high byte
  rv+=Wire.read(); // low byte
  Wire.endTransmission();
  return rv;
}

// ==============================================================
// I2C 7 Segment Display - 4 digit
// ==============================================================
// ==============================================================

// **************************************************************
// Turn relays on
// Input relayNo 1 to 8
// Timed action in seconds 0 to 65000 (0 is now). The actual time
// is arbitary
// **************************************************************
void BV4627_I::on(char relayNo, int timed)
{ 
  if(timed == 0) timed = 1;
  relayNo += 9;
  if((relayNo >= 10 ) && (relayNo <= 17)) {
    Wire.beginTransmission(_i2adr);
    Wire.write(relayNo);  // command
    Wire.write(1); // on
    Wire.write((timed >> 8) & 0xff); // high
    Wire.write(timed  & 0xff); // low
    Wire.endTransmission();
  }
} 
// **************************************************************
// Turn relays off
// Input relayNo 1 to 8
// Timed action in seconds 0 to 65000 (0 is now). The actual time
// is arbitary
// **************************************************************
void BV4627_I::off(char relayNo, int timed)
{ 
  if(timed == 0) timed = 1;
  relayNo += 9;
  if((relayNo >= 10 ) && (relayNo <= 17)) {
    Wire.beginTransmission(_i2adr);
    Wire.write(relayNo);  // command
    Wire.write(0); // off
    Wire.write((timed >> 8) & 0xff); // high
    Wire.write(timed  & 0xff); // low
    Wire.endTransmission();
  }
} 
// **************************************************************
// device ID
// **************************************************************
int BV4627_I::deviceid()
{
  Wire.beginTransmission(_i2adr);
  Wire.write(83);  // device id comand
  Wire.endTransmission();
  return i2_get16bit();
} 
// **************************************************************
// get device firmware version, this is a string!!
// **************************************************************
void BV4627_I::version(char *ver)
{
char b, tmp[20];
  Wire.beginTransmission(_i2adr);
  Wire.write(84);  // firmware comand
  Wire.endTransmission();
  // return as a string in the format x.x
  Wire.requestFrom(_i2adr, 2); // returns 2 bytes
  b=Wire.read(); // major byte
  itoa(b,tmp,10); // convert to string
  *ver=0; // just in case
  strcpy(ver,tmp);
  strcat(ver,".");
  b=Wire.read(); // minor byte
  itoa(b,tmp,10);
  strcat(ver,tmp);
}

// **************************************************************
// reset device
// **************************************************************
void BV4627_I::reset()
{
  Wire.beginTransmission(_i2adr);
  Wire.write(85);  // reset comand
  Wire.endTransmission();
}

